@@ -9,8 +9,9 @@ nodiv_data <- function(phylo, commatrix, coords, proj4string_in = CRS(as.charact
   
   if(!inherits(commatrix, "distrib_data"))
   {
-    if(missing(coords)) stop("if commatrix is not an object of type distrib_data, coords must be specified")
-    dist_dat <- distrib_data(commatrix, coords, proj4string_in, type, shape)
+    if(missing(coords)) 
+      dist_dat <- distrib_data(commatrix, proj4string_in = proj4string_in, type = type, shape = shape) else
+        dist_dat <- distrib_data(commatrix, coords, proj4string_in, type, shape)
   } else dist_dat <- commatrix
   
   if(is.null(dist_dat$species_stats))
@@ -25,6 +26,9 @@ nodiv_data <- function(phylo, commatrix, coords, proj4string_in = CRS(as.charact
     nodiv_dat$node_stats <- dist_dat$nodestats
   
   cat("Comparing taxon names in phylogeny and communities (using picante)\n")
+  
+  colnames(dist_dat$comm) <- match_speciesnames(phylo$tip.label, colnames(dist_dat$comm), do_not_match = TRUE)
+  nodiv_dat$species_stats$species <- colnames(dist_dat$comm)
   temp <- capture.output(dat <- match.phylo.comm(phylo, dist_dat$comm))
   nodiv_dat$phylo <- dat$phy
   nodiv_dat$comm <- dat$comm
@@ -51,10 +55,24 @@ nodiv_data <- function(phylo, commatrix, coords, proj4string_in = CRS(as.charact
   return(nodiv_dat)
 }
 
+is01line <- function(vec){
+  num <- unique(as.numeric(vec))
+  return(!sum(!num %in% 0:1))
+}
+
+BenHoltMatrix <- function(commatrix){
+  last <- sapply(max(1, ncol(commatrix) - 3):ncol(commatrix), function(line) is01line(commatrix[, line]))
+  if(!sum(last) == length(last))
+    return(FALSE)
+  first01 <- sapply(1:min(ncol(commatrix), 30), function(line) is01line(commatrix[, line]))  
+  first01 <- min(which(first01))
+  return(first01)
+}
 
 distrib_data <- function(commatrix, coords = NULL, proj4string_in = CRS(as.character(NA)), type = c("auto", "grid", "points"), shape = NULL)
 {
   type = match.arg(type)
+  cat("Checking input data\n")
   if(inherits(commatrix, "distrib_data")){
     if(is.null(commatrix$species_stats))
       stop("The distrib_data object is from an earlier version of nodiv. Please run update_object on the object before proceeding")
@@ -68,10 +86,26 @@ distrib_data <- function(commatrix, coords = NULL, proj4string_in = CRS(as.chara
     class(ret) <- "distrib_data"
     return(ret)
   }
-  if(is.null(coords))
-    stop("If not commatrix is already of type distrib_data or nodiv_data, coords must be specified")
+  if(is.null(coords)){
+    if(isWorldmapData(commatrix)){
+      cat("Data format identified as Worldmap export file\n")
+      coords <- data.frame(site = paste(commatrix[, 4], commatrix[, 5], sep = '_'), Long = commatrix[, 4], Lat = commatrix[, 5])
+      commatrix <- data.frame(site = coords$site, abu = rep(1, nrow(commatrix)), species = commatrix[, 1])
+      coords <- coords[!duplicated(coords$site), ]
+      if(identical(proj4string_in, CRS(as.character(NA))))
+        proj4string_in <- CRS("+proj=longlat +ellps=WGS84")
+    } else {
+      if((firstnumeric <- BenHoltMatrix(commatrix)) > 1){
+        # this first bit should be moved up under isworldmapmatrix and be used to check for the Ben type of matrix perhaps
+        cat(paste("Commatrix assumed to be a concatenation of coordinates (", firstnumeric - 1," columns) and community matrix\n", sep = ""))
+        coords <- commatrix[, 1:(firstnumeric-1)]
+        commatrix <- commatrix[, -(1:firstnumeric-1)]
+      } else stop("If not commatrix is already of type distrib_data or nodiv_data, a worldmap matrix, or a concatenation of coords and community matrix, coords must be specified")       
+    }     
+  }
+    
   
-  cat("Checking input data\n")
+
   ## Testing that input objects are all right
   if(class(coords) == "SpatialPointsDataFrame" | class(coords) == "SpatialPixelsDataFrame")
     if(!all.equal(proj4string_in,coords@proj4string))
@@ -82,20 +116,28 @@ distrib_data <- function(commatrix, coords = NULL, proj4string_in = CRS(as.chara
 
   if(is.data.frame(commatrix) & ncol(commatrix) == 3 & !is.numeric(commatrix[,3])) #i.e. is the commatrix in phylocom format?
   {
+    cat("Commatrix identified as phylocom format\n")
     commatrix[,1] <- as.character(commatrix[,1])
     commatrix[,3] <- as.character(commatrix[,3])
     commatrix <- sample2matrix(commatrix)   
   }
+
   if(is.data.frame(commatrix)) commatrix <- as.matrix(commatrix)
   if(!is.matrix(commatrix)) stop("commatrix must be a matrix of 0's and 1's, indicating presence or absence")
   if(!is.numeric(commatrix)) stop("commatrix must be a numeric matrix of 0's and 1's, indicating presence or absence")
-  if(!sum(unique(as.numeric(commatrix)) %in% 0:1) == 2) stop("commatrix must be a matrix of 0's and 1's, indicating presence or absence")
+  if(!sum(!unique(as.numeric(commatrix)) %in% 0:1) == 0) stop("commatrix must be a matrix of 0's and 1's, indicating presence or absence")
+  
+  temp <- floor(commatrix) #this is currently not necessary, due to the previous line
+  if(sum(commatrix-temp) > 0)
+    stop("commatrix had non-integer entries, please revise")
   
   if(is.matrix(coords)) coords <- as.data.frame(coords)
+  cat("Transforming coords to spatial points\n")
   if(is.data.frame(coords)) coords <- toSpatialPoints(coords,proj4string_in, commatrix, type)
 
   if(class(coords) == "SpatialPixelsDataFrame") type <- "grid" else if (class(coords) == "SpatialPointsDataFrame") type <- "points" else stop("coords must be a data.frame of coordinates or an sp data.frame object")
   
+
   ## making sure that the points and the commatrix fit
   
   
@@ -118,6 +160,9 @@ distrib_data <- function(commatrix, coords = NULL, proj4string_in = CRS(as.chara
   }
   
   ret <- list(comm = as.data.frame(commatrix), type = type, coords = coords)
+  if(type == "grid")
+    ret$grid <- summary(ret$coords)$grid 
+  
   ret$species_stats <- data.frame(species = colnames(ret$comm), stringsAsFactors = FALSE)
   
   if(!is.null(shape)) ret$shape <- shape
@@ -167,6 +212,7 @@ match_commat_coords <- function(commatrix, sitenames)
 
 toSpatialPoints <- function(coords, proj4string, commatrix, type)
 {
+  
     xcol <- 0
     ycol <- 0
     
@@ -177,34 +223,51 @@ toSpatialPoints <- function(coords, proj4string, commatrix, type)
     {
       xcol <- which(colnames(ret) == 'x')
       ycol <- which(colnames(ret) == 'y')
-      ret <- ret[,c(xcol, ycol)]
       
     } else if('lon' %in% substr(colnames(ret),1,3) & 'lat' %in% substr(colnames(ret), 1, 3)) {
-      
       colnames(ret) <- substr(colnames(ret), 1, 3)
       xcol <- which(colnames(ret) == 'lon')[1]
       ycol <- which(colnames(ret) == 'lat')[1]
-      ret <- ret[,c( xcol, ycol)]
     }
+
+    names(coords)[xcol] = "myX"
+    names(coords)[ycol] = "myY"
     
-    if(!ncol(ret) == 2) stop("ret should be a data.frame or spatial data.frame with 2 columns, giving the x/longitude, and y/latitude of all sites")
-      
-    ret <- SpatialPoints(ret, proj4string)
+    cat("Identifying sites identifier\n")
     
-    if (ncol(coords)==3 & !(xcol + ycol == 0) & isTRUE(all.equal(coords[,-c(xcol, ycol)], unique(coords[,-c(xcol, ycol)])))) sitenames <- coords[,-c(xcol, ycol)] else 
-      if(nrow(coords) == nrow(commatrix) & !is.null(rownames(commatrix))) sitenames <- rownames(commatrix) else
-        if(!is.null(rownames(coords))) sitenames <- rownames(coords) else 
+    if (ncol(coords)==3 & !(xcol + ycol == 0) & isTRUE(all.equal(coords[,-c(xcol, ycol)], unique(coords[,-c(xcol, ycol)])))) names(coords)[!names(coords) %in% c("myX", "myY")] <- "sites" else 
+      if(nrow(coords) == nrow(commatrix) & !is.null(rownames(commatrix)) & ncol(coords) == 2){
+        if(is.null(rownames(coords)))
+          coords = data.frame(sites = rownames(commatrix)) else 
+            if (!identical(rownames(commatrix), rownames(coords))) stop("Because the rownames of commatrix and coords differ, sitenames cannot be established unless they are included explicitly as a third column of coords")
+      }  else {
+        if(is.null(rownames(commatrix))){
           stop("There must be valid site names in the rownames of commatrix or in the coords data")
+        } else {
+          coords <- infer_sites_intern(rownames(commatrix), coords)
+        }
+      }
     
-    type_auto <- ifelse(isGrid(ret), "grid", "points")
+    if(sum(names(coords) == "sites") > 1)
+      stop(paste("Could not match on the variable called sites, please rename"))
+    
+    coords$sites <- as.character(coords$sites)
+    ids <- c(which(names(coords) == "myX"), which(names(coords) == "myY") )
+    xy <- coords[, ids]    
+    if(!ncol(xy) == 2) stop("coords should be a data.frame or spatial data.frame with 2 columns, giving the x/longitude, and y/latitude of all sites")
+    
+    xy <- SpatialPoints(xy, proj4string)
+    type_auto <- ifelse(isGrid(xy), "grid", "points")
     
     if(type == "auto") type <- type_auto else 
       if(!type == type_auto)
-        warning("The specified type of data (points or grid) seems to conflict with the automatic setting. This may cause problems")
+        warning(paste("The specified type of data (", type, ") seems to conflict with the automatic setting. This may cause problems", sep = ""))
     
-    
-    if(type == "grid") ret <- SpatialPixelsDataFrame(ret, data.frame(sites = sitenames, stringsAsFactors = F)) else
-      ret <- SpatialPointsDataFrame(ret, data.frame(sites = sitenames, stringsAsFactors = F))
+    if(ncol(coords) == 3)
+      ret <- data.frame(sites = coords[, - ids], stringsAsFactors = FALSE) else
+        ret <- data.frame(coords[, -ids], stringsAsFactors = FALSE)
+    if(type == "grid") ret <- SpatialPixelsDataFrame(xy, ret) else
+      ret <- SpatialPointsDataFrame(xy, ret)
     
     return(ret)  
 }
@@ -222,18 +285,25 @@ isGridVar <- function(gridVar)
   #if all differences are a multiplum of the smallest, and the smallest distance is the most common, it is probably a grid
 }
 
+
 Create_node_by_species_matrix = function(tree)
 {
   # create a matrix with 0s and 1s indicating which species descend from each node
   nodespecies <- matrix(0, nrow = Nnode(tree), ncol = Ntip(tree))
   colnames(nodespecies) <- tree$tip.label
   rownames(nodespecies) <- nodenumbers(tree)
-  
  
-  for ( i in 1:Nnode(tree))
+  ntip <- Ntip(tree)
+  .local <- function(tree, node)
   {
-    nodespecies[i,Node_spec(tree, nodenumbers(tree)[i], names = FALSE)] <- 1
+    if(node <= Ntip(tree))
+      return(node)
+    ret <- lapply(Descendants(node, tree), .local, tree = tree)
+    ret <- do.call(c, ret)
+    nodespecies[node - ntip, ret] <<- 1
+    ret
   }
+  .local(tree, basal_node(tree))
   
   return(nodespecies)
 }
@@ -248,8 +318,6 @@ Node_spec <- function(tree, node, names = TRUE)
     do.call(c, ret)
   }
   
-  if(inherits(tree, "nodiv_data"))
-    tree <- tree$phylo
   if(!inherits(tree, "phylo"))
     stop("tree must be an object of type phylo or nodiv_data")
   
@@ -262,4 +330,18 @@ Node_spec <- function(tree, node, names = TRUE)
   ret
 }
 
+isWorldmapData <- function(dat){
+  if(is.data.frame(dat)){
+    if(is.factor(dat[, 1]))
+      dat[, 1] <- as.character(dat[, 1])
+    if(ncol(dat) == 5)
+      if(is.character(dat[, 1]))
+        if(is.numeric(dat[, 4]))
+          if(is.numeric(dat[, 5]))
+            if(min(dat[, 4], na.rm = T) > -181 & max(dat[, 4], na.rm = T) < 181)
+              if(min(dat[, 5], na.rm = T) > -91 & max(dat[, 5], na.rm = T) < 91)
+                return(TRUE)
+  }
+  return(FALSE)
+}
 
